#' Function to compute what fraction of grid cells lies within  the given polygon
#' 
#' This function computes for a regular grid (one vector of longitudes and latitudes), what fraction of each grid cell lies within the given polygon.
#' 
#' @param lon A vector of grid longitudes
#' @param lat A vector of grid latitudes
#' @param poly.x The longitudes of the edges of the polygon
#' @param poly.y The latitudes of the edges of the polygon
#' @param multiply Factor by which the grid is inflated for computation
#'
#' @return Returns a matrix of dimension nlon x nlat of fractional gridbox area within the polygon
#' 
#' @details The accuracy for this method scales with the multiplying factor. After inflation, grid points within the polygon are found with \code{point.in.polygon} from the package \code{sp}
#'
#' @author Jonas Bhend
#' 
#' @seealso \code{\link[sp]{point.in.polygon}}
#' 
#' @examples
#'    # set up latitudes and longitudes
#'    lon       <- seq(-3.5,3.5,0.5)
#'    lat       <- seq(-3.5,3.5,0.5)
#'    # polygon is a circle with radius 3
#'    poly.x    <- 3*sin(seq(0,2*pi,length=100))+0.13
#'    poly.y    <- 3*cos(seq(0,2*pi,length=100))-0.32
#'    outgrid   <- frac_in_polygon(lon,lat,poly.x,poly.y, multiply=5)
#'    # plot the result
#'    image(lon,lat,outgrid, asp=1)
#'    lines(poly.x, poly.y, lwd=2)
#'
#' @export
`frac_in_polygon` <- 
function(lon, lat, poly.x, poly.y, multiply=4){

    # blow up longitudes
    nlon    <- length(lon)
    lon.alt <- c(lon[1] - 0.5*diff(lon[1:2]), lon, 
        lon[nlon] + 0.5*diff(lon[nlon - 1:0]))
    lon.ind <- c(0,seq(0.5/nlon, 1-0.5/nlon,length=nlon),1)
    ind.out <- seq(0.5/nlon/multiply, 1- 0.5/nlon/multiply, length=nlon*multiply)
    lon.new <- approx(lon.ind, lon.alt, ind.out)$y
    
    # blow up latitudes
    nlat    <- length(lat)
    lat.alt <- c(lat[1] - 0.5*diff(lat[1:2]), lat, 
        lat[nlat] + 0.5*diff(lat[nlat - 1:0]))
    lat.ind <- c(0,seq(0.5/nlat, 1-0.5/nlat,length=nlat),1)
    ind.out <- seq(0.5/nlat/multiply, 1- 0.5/nlat/multiply, length=nlat*multiply)
    lat.new <- approx(lat.ind, lat.alt, ind.out)$y
    
    # find points in polygon
    outgrid <- point.in.polygon(rep(lon.new, length(lat.new)),
        rep(lat.new, each=length(lon.new)), poly.x, poly.y)
    if (multiply == 1){
        outgrid[outgrid > 1] <- NA
    } else {
        outgrid[outgrid > 1] <- 0.5
    }
    
    out.tmp <- array(outgrid, c(multiply, nlon, multiply, nlat))
    fracout <- apply(out.tmp, c(2,4), mean, na.rm=T)
    
    fracout
}
