#' Function to convert geographical into model coordinates
#' 
#' Functions to converts geographical latitudes and longitudes to positions on a regular model grid.
#' 
#'
#' @param lon geographical longitudes of the model grid points, a vector or a matrix
#' @param lat geographical latitudes of the model grid points, a vector or a matrix
#' @param ptlon geographical longitude values to be converted, a vector with the same length as ptlat
#' @param ptlat geographical longitude values to be converted, a vector with the same length as ptlon
#' @param gridmapping A list of attributes for the grid mapping as it is returned by the plotmap function
#' 
#' @return   Returns a list with two vectors:
#'    \item{x}{Processed longitudes}
#'    \item{y}{Processed latitudes}
#'
#' @note This function does not work for unstructured grid
#' 
#' @author Burkhardt Rockel (HZG)
#'
#' @examples
#'    # for rotated coordinates
#'
#'    tmp <- plotmap(...)
#'    point<- geo2mod(lon, lat, c(10.,30.,-40.), c(50.,60.,60.),gridmapping=tmp$gridmapping)
#'
#'    # for all other coordinates
#'    tmp <- plotmap(...)
#'    point<- geo2mod(lon, lat, c(10.,30.,-40.), c(50.,60.,60.),gridmapping=list(attribute1=NULL))
#'
#' @export
geo2mod <- function(lon,lat,ptlon,ptlat, gridmapping=list(attribute1=NULL)) {

  npt = length(ptlon)
#  if (is.null(gridmapping))  gridmapping<-list(attribute1=NULL)
  if ((is.null(ncol(lon)) || is.na(ncol(lon))) && is.null(gridmapping$attribute1) ) {
    #... lat/lon are 1D. Therefore assuming geographical lat/lon. No conversion necessary in this case.
    out     <- list(x=ptlon, y=ptlat)

  } else if ( !is.null(gridmapping$attribute1) ) { 
     #... data are in rotated coordinates
     out <- geo2rot(gridmapping$attribute1,gridmapping$attribute2,ptlon, ptlat, gridmapping$attribute3, gridmapping$attribute4)
  } else {

    #... lon/lat are 2D. Grid mapping is assumed in this case.
    if (!is.loaded('find_closest')) {
       dyn.load('/Volumes/Moritz/R/new_plotmap/find_closest.so')
    }  
    nx = dim(lon)[1]
    ny = dim(lon)[2]

    ptlon[is.na(ptlon)] <- -999
    ptlat[is.na(ptlat)] <- -999
    undef <- -999
  
    retdata <- .Fortran("find_closest",
                       undef = as.double(undef),
                       ptlon = as.double(ptlon),
                       ptlat = as.double(ptlat),
                       lon = as.double(lon), 
                       lat = as.double(lat), 
                       npt = as.integer(npt), 
                       nx = as.integer(nx), 
                       ny = as.integer(ny), 
                       xindex = double(npt),
                       yindex = double(npt))
        out     <- list(x=retdata$xindex-1, y=retdata$yindex-1)
  }
}

