#' Function to extract dates and times from NetCDF file opened with nc_open()
#' 
#' This function extracts the date and time information from NetCDF files that follow the CF conventions and converts it either to the internal Date format.
#' 
#' @param nc The connection to the netcdf file as with nc_open()
#' 
#' @return A vector of Dates or Datetimes depending on the units in the NetCDF file.
#' 
#' @note his function assumes that the netCDF is in CF convention, ie. the variable holding the time information must be named "time".
#'    The time variable must have a "units" attribute in the form "seconds since YYYY-MM-DD HH:MM:SS" where "seconds" can also be "minutes", "hours", "days", "months", "years"
#'
#' @author Burkhardt Rockel (HZG)
#' @seealso {\code{\link[ncdf4]{nc_open}}, \code{\link{as.PCICt}}}
#' @examples
#'     # Open a file and read its data
#'     nc        <- nc_open("foo.nc")
#'     times     <- nccf_times(nc)
#'     print("here is the data in the file:")
#'     head(times) 
#'     tail(times)
#'     t2m       <- ncvar_get(nc, "T_2M")
#'     nc_close( nc )
#'     plot(times, t2m, type="l")
#'  
#' @export
`nccf_times` <-
function(nc) {
    # this function converts netcdf times in a netCDF file 
    # with CF conventions to the 
    # R date-time format
        
    timei   <- which(names(nc$dim) %in% c("time", "TIME", "tim", "TIM"))
    units   <- nc$dim[[timei]]$units
    vals    <- nc$dim[[timei]]$vals
    tmp     <- ncatt_get(nc, names(nc$dim)[timei], "calendar")
    if (tmp$hasatt) {
        calendar <- trimws(tmp$value)
    } else {
        calendar <- "standard"
        print(paste("Warning: Calendar is missing in", nc$filename))
        print(paste('         "standard" calendar is used'))
    }    
#    if (calendar == "proleptic_gregorian") calendar <- "standard"
#... valid calendar names
    cals <- c("standard", "gregorian", "proleptic_gregorian", "360_day", "360", "365_day", "365", "noleap")
    itype.cals <- c(0, 0, 0, 1, 1, 2, 2, 2)
    if (identical(calendar %in% cals, FALSE)){
        print(paste("Warning: ",calendar," is not a valid calendar name" ))
        print(paste("valid names are: ", cals))
        print(paste('         "standard" calendar is used'))     
        calendar <- "standard"
    }

    units.dt <- c("seconds since", "minutes since", "hours since","days since","months since","years since")
    mul.dt <- c(1, 60, 3600, 86400, 1, 1)
#... find the index of matching units.td in units
    index.dt <- match(1,sapply (units.dt,function(x) charmatch(x, units, nomatch=0)),nomatch=0)
    if (index.dt == 0) {
         print(paste("ERROR: ",units,"is no valid unit"))
         print(paste("accepted units are:"))
         print(units.dt)
         stop
#         quit()
    }

    ref.time     <- trimws(gsub(units.dt[index.dt],"",units))
    ref.time.pcict <- as.PCICt(ref.time, calendar)

    if (index.dt == 5) {
      ymd<-format(as.Date(ref.time), "%Y-%m-01")
      ref.time.pcict.month<-as.PCICt(ymd,calendar)
      y<-sapply (vals,function(x) seq(ref.time.pcict.month, by = "month", length = x+1))
      z<-lapply(seq(1,length(y),1), function(x) tail(y[[x]],-(length(y[[x]])-1)))
      zz<-do.call(c,z)
      zzz<-as.PCICt(zz, calendar) - ref.time.pcict.month
      vals<-do.call(c,lapply(seq(1,length(zzz),1),function(x) zzz[[x]]))
    }

    if (index.dt == 6) {
      ymd<-format(as.Date(ref.time), "%Y-01-01")
      ref.time.pcict.year<-as.PCICt(ymd,calendar)
      y<-sapply (vals,function(x) seq(ref.time.pcict.year, by = "year", length = x+1))
      z<-lapply(seq(1,length(y),1), function(x) tail(y[[x]],-(length(y[[x]])-1)))
      zz<-do.call(c,z)
      zzz<-as.PCICt(zz, calendar) - ref.time.pcict.year
      vals<-do.call(c,lapply(seq(1,length(zzz),1),function(x) zzz[[x]]))
    }

    mul<-mul.dt[index.dt]
    times       <- vals * mul
    time <- ref.time.pcict + times

    time

}

