#' Function to plot a colourbar
#' 
#' Generic function to plot a horizontal or vertical colourbar with axis on the specified side.
#' 
#' @param levs Levels (boundaries)
#' @param cols colours to use
#' @param x.list A list of class "plotmap" containing the levels and colours and additional information as returned by \code{plotmap}
#' @param incl.units allows for the long-name and units attribute of the plotted variable to be added to the axis annotation)
#' @param nn Each nn axis label will be drawn
#' @param side At which side of the image the colour bar will be plottet: 1=bottom 2=left, 3=top, 4=right
#' @param cex.axis The magnification to be used for axis annotation relative to the current setting of "cex""
#' @param center If TRUE the labels will be drawn in the center beside the colors
#' @param labels If not NULL it overwrites the default labels given by levels in x.list
#' @param border Colour of the border around the colour bar and the axis (but not ticks)
#' @param triangle_ends If not NULL it must be a logical vector with two elements. If an element is TRUE a triangle is plottet instead of a rectangle at the end of the color bar. First element gives the values for the start of the colour bar, the second element for the end of the colour bar
#'
#' @return Nothing to return
#' 
#' @details   There must be one more levels than colours including the colour for water (\code{sea.col}). 
#'    The enumeration of side is consistent with \code{axis()}, e.g. from 1 to 4 clockwise from bottom.
#'    Only every nn'th label is drawn. 
#'    Ticks are omitted if centered labels are drawn.
#'    
#' @author Jonas Bhend, Burkhardt Rockel (HZG)
#'
#' @seealso \code{\link{plotmap}}
#' 
#' @examples
#'    tmp <- plotmap(file, ...)
#'    plot_colourbar(tmp, triangle_ends=c(TRUE,FALSE),border=NA)
#'    
#' @export
`plot_colourbar` <-
function (x, ...){
    UseMethod("plot_colourbar")
}

#' @describeIn plot_colourbar Default, plots a colour bar given the levs and cols
`plot_colourbar.default` <-
function(levs, cols, side=1, ylab="", labels=NULL,
    xlab="", nn=1, center=F, cex.axis=1, sea.col=NULL, sea.lab=NULL, border=par("fg"), triangle_ends=NULL, ...){
    # plots a colour bar given the levs and cols
    # centers the axis labelling instead of at the 
    # boundary when center==TRUE
    # if sea.col is set, the colourbar is prolonged by one
    # colour with the uncentered label sea underneath
      # Check triangle_ends
  if (!is.null(triangle_ends) && (!is.logical(triangle_ends) || length(triangle_ends) != 2)) {
    stop("Parameter 'triangle_ends' must be a logical vector with two elements.")
  }

    las<-c(1,3,1,3)

    sea.add     <- 0
    if (!is.null(sea.col)) {
        cols    <- c(sea.col, cols)
        sea.add <- 1
    }
    ncols       <- length(cols)
    lev.arr     <- array(1:ncols, c(1, ncols))

    triangles.allowed<-FALSE
    if (sum(triangle_ends) > 0 && !center && sea.add==0) triangles.allowed<-TRUE

    #... colorbar at bottom side=1 or top side=3
    if (side %in% c(1,3)){
        lev.arr <- t(lev.arr)
        if (triangles.allowed) {
        #... triangles at the ends
          image(1:ncols, 1, lev.arr, axes=F, breaks=1:(ncols+1)-0.5, col=rep(NA,ncols), 
                         ylab=NA, xlab=NA, ...)
          usr <- par( "usr")
          triangle_height <- (usr[2]-usr[1])/ncols
          left_triangle <- list(x = c(usr[1], usr[1]+triangle_height, usr[1]+triangle_height) ,
                                y = c(usr[3]+(usr[4]-usr[3])/2, usr[4], usr[3]))
          right_triangle <- list(x = c(usr[2], usr[2]-triangle_height, usr[2]-triangle_height) ,
                                 y = c(usr[3]+(usr[4]-usr[3])/2, usr[4], usr[3]))
          if (sum(triangle_ends) == 2) {
          #... triangles at both ends
            col.start <- 2
            col.end <- ncols-1
            lab.start<-2
            lab.end<-ncols
            image(1:ncols, 1, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(NA,cols[col.start:col.end],NA),
               ylab=ylab, xlab=xlab, ...)
            abline(v=col.start:col.end + 0.5, col=border)
            polygon(left_triangle$x, left_triangle$y,col=cols[1],border=border)
            polygon(right_triangle$x, left_triangle$y,col=cols[ncols],border=border)
          } else if (triangle_ends[1]) {
          #... triangle at the left end
            col.start <- 2
            col.end <- ncols
            lab.start<-2
            lab.end<-ncols+1
            image(1:ncols, 1, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(NA,cols[col.start:col.end]),
               ylab=ylab, xlab=xlab, ...)
            abline(v=col.start:col.end + 0.5, col=border)
            polygon(left_triangle$x, left_triangle$y,col=cols[1],border=border)
          } else {
          #... triangle at the right end
            col.start <- 1
            col.end <- ncols-1
            lab.start<-1
            lab.end<-ncols
            image(1:ncols, 1, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(cols[col.start:col.end],NA),
               ylab=ylab, xlab=xlab, ...)
            abline(v=col.start:col.end + 0.5, col=border)
            polygon(right_triangle$x, left_triangle$y,col=cols[ncols],border=border)
          }
        } else {
       #... no triangles at the ends

            image(1:ncols, 1, lev.arr, axes=F, breaks=1:(ncols+1)-0.5, col=cols,
            ylab=ylab, xlab=xlab, ...)
       if (!is.na(border)) abline(v=1:(ncols-1) + 0.5, col=border)
        } 
    } else {
    #... colorbar left side=2 or right side=4
        if (triangles.allowed) {
        #... triangles at the ends
          image(1, 1:ncols, lev.arr, axes=F, breaks=1:(ncols+1)-0.5, col=rep(NA,ncols), 
                         ylab=NA, xlab=NA, ...)
          usr <- par( "usr")
          triangle_height <- (usr[4]-usr[3])/ncols
          bottom_triangle <- list(x = c(usr[1]+(usr[2]-usr[1])/2, usr[2],usr[1]) ,
                                y = c(usr[3], usr[3]+triangle_height, usr[3]+triangle_height))
          top_triangle <- list(x = c(usr[1]+(usr[2]-usr[1])/2, usr[2],usr[1]) ,
                                 y = c(usr[4], usr[4]-triangle_height, usr[4]-triangle_height))

          if (sum(triangle_ends) == 2) {
          #... triangles at both ends
            col.start <- 2
            col.end <- ncols-1
            lab.start<-2
            lab.end<-ncols
            image(1, 1:ncols, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(NA,cols[col.start:col.end],NA),
               ylab=ylab, xlab=xlab, ...)
            abline(h=col.start:col.end + 0.5, col=border)
            polygon(bottom_triangle$x, bottom_triangle$y,col=cols[1],border=border)
            polygon(top_triangle$x, top_triangle$y,col=cols[ncols],border=border)
          } else if (triangle_ends[1]) {
          #... triangle at the bottom end
            col.start <- 2
            col.end <- ncols
            lab.start<-2
            lab.end<-ncols+1
            image(1, 1:ncols, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(NA,cols[col.start:col.end]),
               ylab=ylab, xlab=xlab, ...)
            abline(h=col.start:col.end + 0.5, col=border)
            polygon(bottom_triangle$x, bottom_triangle$y,col=cols[1],border=border)
          } else {
          #... triangle at the right end
            col.start <- 1
            col.end <- ncols-1
            lab.start<-1
            lab.end<-ncols
            image(1, 1:ncols, lev.arr, axes=F, add=TRUE, breaks=1:(ncols+1)-0.5, col=c(cols[col.start:col.end],NA),
               ylab=ylab, xlab=xlab, ...)
            abline(h=col.start:col.end + 0.5, col=border)
            polygon(top_triangle$x, top_triangle$y,col=cols[ncols],border=border)
          }
        } else {
       #... no triangles at the ends

          image(1, 1:ncols, lev.arr, axes=F, breaks=1:(ncols+1)-0.5, col=cols,
              ylab=ylab, xlab=xlab, ...)
          if (!is.na(border)) abline(h=1:(ncols-1) + 0.5, col=border)
       }
    }


    if (center){
        at.lev  <- seq(1+sea.add,ncols,nn)
        if (is.null(labels)){
            labels <- (levs[1:(ncols-sea.add)] + levs[2:(ncols+1-sea.add)])/2
        }
        axis(side, at=at.lev, labels=labels, las=1, tick=F, cex.axis=cex.axis)
    } else {
      if (triangles.allowed) {
         at.lev  <- seq(lab.start,lab.end,nn)
       if (is.null(labels)){
            labels  <- levs[at.lev]
        }     
      } else {
        at.lev  <- seq(2+sea.add,ncols,nn)
        if (is.null(labels)){
            labels  <- levs[at.lev - sea.add]
        }
      }
 #       if (!is.na(border)) {
 #         axis(side, at=at.lev-0.5,labels=labels, las=1, cex.axis=cex.axis)
 #       } else {
 #         axis(side, at=at.lev-0.5,labels=labels, las=1, col = NA, col.ticks = 1, cex.axis=cex.axis)
          axis(side, at=at.lev-0.5,labels=labels, las=1, col = border, col.ticks = 1, cex.axis=cex.axis)
 #       }
    }
    if (!is.null(sea.col)){
        axis(side, at=1, labels=sea.lab, las=las[side], tick=F, cex.axis=cex.axis)
    }
 if (triangles.allowed) {
    mai.bak <- par("mai")
    mai<-mai.bak
    pin <- par("pin")
    if (side %in% c(1,3)){
      dx<-pin[1]/ncols
      if (triangle_ends[1]) mai[2]<-mai.bak[2]+dx
      if (triangle_ends[2]) mai[4]<-mai.bak[4]+dx
    } else {
      dy<-pin[2]/ncols
      if (triangle_ends[1]) mai[1]<-mai.bak[1]+dy
      if (triangle_ends[2]) mai[3]<-mai.bak[3]+dy
    }
    par(mai=mai)
  }
    if (!is.na(border)) box(col=border)
    if (triangles.allowed) par(mai=mai.bak)

}

#' @describeIn plot_colourbar Color bar for plotmap
`plot_colourbar.plotmap` <-
function(x, incl.units=T, side=1, cex.axis=1, labels=NULL, center=F, ...){
    if (!is.null(x$flag_values)){
        center  <- TRUE
        labels  <- x$flag_values
        if (!is.null(x$flag_meanings)){
            labels  <- x$flag_meanings
        }
    }
    las<-c(1,3,1,3)
    plot_colourbar(x$lev, x$col, sea.col=x$sea.col, sea.lab=x$sea.lab, side=side,
        cex.axis=cex.axis, labels=labels, center=center, ...)

    # plot unit-string in axis
    if (incl.units){
        if (!is.null(x$longname)) {
            if (!is.null(x$units)){
                unit.string <- paste(x$longname," (",x$units, ")", sep="")
            } else {
                unit.string <- x$longname
            }
            nlev    <- length(x$lev)            
            axis(side, at=nlev/2, labels=unit.string, 
                tick=F, las=las[side], line=2, cex.axis=cex.axis)
        } else {
            if (!is.null(x$units)){
                axis(side, at=length(x$lev)-0.5, labels=x$units, 
                    tick=F, las=1, cex.axis=cex.axis)
            }
        }
    }
}

