#' Plot NetCDF data on rotated, multiple rotated or regular lonlat grids
#'
#' This function reads in NetCDF data and produces a plot of the specified variable.
#'
#' @param file    Path to the NetCDF file containing the data to plot
#' @param sponge  Number of grid cells to mask out at boundary
#' @param varname Variable to plot
#' @param lsm.file Additional file containing land-sea mask and/or altitude 
#'    information in quantities with the \code{standard_names}: 
#'    \code{land_area_fraction} and \code{surface_altitude}
#' @param units    overwrites the units from the netCDF file
#' @param longname overwrites the longname from the netCDF file
#' @param add.offset adds a constant to the quantity to be plotted
#' @param scale.factor multiplies the quantity to be plotted with a constant
#' @param col Colours to use
#' @param levels Levels to use
#' @param sea.col Colour for masked out water grid points
#' @param sea.lab Label for the water on the color bar
#' @param na.col Colour for grid points with undefined values
#' @param rivers Switch for automatically added rivers
#' @param lakes Switch for automatically added lakes
#' @param cities Switch for automatically added cities
#' @param label Switch for labels of cities
#' @param minpop Minimal population for cities to get drawn
#' @param ncities Number of cities to draw
#' @param city.pch Symbol for city points
#' @param alt.contour Switch for usage of the coastline as in file or lsm.file
#' @param alt.lev Level at which to draw coastline (if alt.contour == TRUE) in
#'    units of \code{"FR_LAND"}
#' @param grid Switch for inclusion of grid lines
#' @param grid.txt Switch for inclusion of grid axis labels at the border of 
#'    the plot. Can be set for each side separately by using grid.txt=c(T,T,T,T)
#' @param grid.lty Line type for grid lines
#' @param grid.lwd Line width for grid lines
#' @param i.time Time step to plot
#' @param i.lev Level to plot
#' @param map.lwd Line width for the coastline and country borders
#' @param map.lty Line type for the coastline and country borders
#' @param cex.axis Character expansion for axis lat/lon labels
#' @param cex.lab Character expansion for  x and y labels
#' @param cex.main Character expansion for main titles
#' @param cex.txt Character expansion for city labels
#' @param add Should plot be added to existing plot
#' @param colourplot Switch for colour or grayscale plot
#' @param hires Use hi-resolution coastline
#' @param interior Draw political boundaries
#' @param alt.poli Switch to use alternative political boundaries
#' @param nlongrid  Approximate number of longitude grid lines 
#' @param nlatgrid  Approximate number of latitude grid lines
#' @param lon.ind Sequence of longitudes at which to 
#'    draw grid lines. Has to be increasing values. Longitudes greater than 180 are 
#'    converted.
#' @param lat.ind Sequence of latitudes at which to 
#'    draw grid lines. Has to be increasing values.
#' @param pollonshift Effects only special cases where pollat=90. in rotated
#'    coordinates. For example for rotated==geographic coordinate system 
#'    pollon=0., pollat=90.. However, due to a bug in nearly all existing 
#'    conversion routines, this is mostly defined as pollon=180., pollat=90. 
#'    or pollon=0., pollat=90., polgam=180.. In the latter case set 
#'    pollonshift=T
#' @param smoothed Smooth the plot. Internally use of \code{filled.contour} 
#'    instead of \code{image}
#' @param geo.coordinate.names names fo the 2D fields of the geographical 
#'    longitude and latitude at the model grid points
#' 
#' @return A List of class "plotmap" containing the following objects:
#' \item{lev}{The levels used}
#' \item{col}{The colours used}
#' \item{sea.col}{The colour used for water grid points}
#' \item{sea.lab}{Label for the water on the color bar}
#' \item{flag_values}{Flag values taken from the netCDF attribute flag_values, if available}
#' \item{flag_meanings}{Flag meanings taken from the netCDF attribute flag_meanings, if available}
#' \item{longname}{The longname attribute of the variable plotted}
#' \item{units}{The units of the variable plotted}
#' \item{gridmapping}{A list containing the grid mapping attributes and additionals. For example rotated pole coodinates: gridmapping=list(attribute1=pollon, attribute2=pollat, attribute3=polgam, attribute4=pollonshift) }
#'
#' @note This function should works for geographical and rotated
#'     model coordinates and for most structured grids, if additional geographic lat/lon data are provided for the model grid.
#'     It is possible to plot icosahedral output. The geographical longitudes and latitudes of the vertices have to be given in variables with the names \code{clon_bnds} and \code{clat_bnds} or \code{clon_vertices} and \code{clat_vertices}.
#' @examples 
#'    tmp<-get.ethz.palette("kd_temp_1m")
#'    col<-tmp$colors
#'    levs<-tmp$levels
#'    out <- plotmap(...,col=col,levels=levs, ...)
#' 
#' @export

plotmap <-
function(file,  sponge=0, 
    varname = NULL, lsm.file=NULL,  
    units=NULL, longname=NULL, add.offset=0, scale.factor=1,
    col=NULL, levels=NULL, sea.col=NULL, sea.lab="water", na.col=NULL, rivers=T, lakes=T, 
    cities=T, label=TRUE, minpop=NULL, ncities=10, city.pch=19,
    alt.lev=NULL,
    grid=TRUE, grid.txt=c(T,T,T,T), grid.lty=2, grid.lwd=1,
    i.time=1, i.lev=1, map.lwd=2, map.lty=1,
    cex.axis=1, cex.lab=1, cex.main=1, cex.txt=1,
    main="", xlab="", ylab="", add=FALSE,
    colourplot=TRUE, hires=FALSE, interior=FALSE, alt.poli=TRUE,
    nlongrid=10, nlatgrid=5, lon.ind, lat.ind, pollonshift=TRUE,smoothed=FALSE,geo.coordinate.names=c("lon","lat"),alt.contour=FALSE){
    
    # help variable for the ICON model
     icon<-FALSE
    # load hi-resolution map-data only when hires is set
    # in order to avoid complications on a system without
    # the mapdata package
    if (hires) library("mapdata")
    gridmapping<-list(attribute1=NULL)
    #---------------------------------------------------------   
    # read in the data from file
    nc <- nc_open(file)
    if ( is.null(varname) ) stop ('ERROR: varname not defined')
 
    if ( !is.null(nc$var$rotated_pole$id) ) {
     #... data are in rotated coordinates
        pollon  <- as.numeric(ncatt_get(nc, "rotated_pole", 
          "grid_north_pole_longitude")$value)
        pollat  <- as.numeric(ncatt_get(nc, "rotated_pole", 
          "grid_north_pole_latitude")$value)
         polgam  <- ncatt_get(nc, "rotated_pole", 
          "north_pole_grid_longitude")
        if (polgam$hasatt){
          polgam <- polgam$value
       } else {
          polgam <- 0
       }
       gridmapping<-list(attribute1=pollon, attribute2=pollat, attribute3=polgam, attribute4=pollonshift)
       mlon    <- nc$dim$rlon$vals
       mlat    <- nc$dim$rlat$vals
       lon.ext <- mlon
       lat.ext <- mlat
       nlon    <- length(mlon)
       nlat    <- length(mlat)
      if (any(names(nc$var) %in% c("lon", "lat"))){
        lon <- ncvar_get(nc, "lon")
        lat <- ncvar_get(nc, "lat")
      } else {
        tmp <- rot2geo(pollon, pollat, rep(mlon, nlat), rep(mlat, each=nlon), polgam, pollonshift)
            lon <- array(tmp$x, c(nlon,nlat))
            lat <- array(tmp$y, c(nlon,nlat))
      }
 
 # } else if (ncatt_get(nc, varname, "coordinates")$hasatt && ncatt_get(nc, varname, "coordinates")$value == "clat clon") {
  } else if (!is.null(nc$var$clon$id) && !is.null(nc$var$clat$id)) {


    icon<-TRUE
    if (sponge > 0) {
      warning(paste("Masking out the sponge layer is not yet supported for ICON grid!",sep=""))
      sponge <- 0 # masking out of sponge not yet supported for ICON grid
    }
    print ("ICON output")
    lon.vertices.name <- 'clon_bnds'
    lat.vertices.name <- 'clat_bnds'
    if ( is.null(nc$var$clon_bnds$id) ) {
       lon.vertices.name <- 'clon_vertices'
      if ( is.null(nc$var$clon_vertices$id) ) {
        stop(paste("ERROR: neither variable clon_bnds nor clon_vertices found",sep=""))
      }
    }
    if ( is.null(nc$var$clat_bnds$id) ) {
        lat.vertices.name <- 'clat_vertices'
     if ( is.null(nc$var$clat_vertices$id) ) {
        stop(paste("ERROR: neither variable clat_bnds nor clat_vertices found",sep=""))
      }
    }
    clon_bnds<-ncvar_get(nc,lon.vertices.name)
    clat_bnds<-ncvar_get(nc,lat.vertices.name)
    vlon <- clon_bnds * 180./pi
    vlat <- clat_bnds * 180./pi
    vlon.range<-c(min(vlon),max(vlon))
    vlat.range<-c(min(vlat),max(vlat))
    vmod.x <- vlon
    vmod.y <- vlat
    vmod.xrange <- vlon.range
    vmod.yrange <- vlat.range
#    print (vmod.xrange)
#    print (vmod.yrange)
    nmod<-100
    xyratio<-(max(vmod.x)-min(vmod.x))/(max(vmod.y)-min(vmod.y))
#    print(paste("xyratio ",xyratio,sep=" "))
    if (xyratio > 1) {
      nx <- as.integer(nmod * xyratio)
      ny <- nmod
    } else {
      nx <- nmod
      ny <- as.integer(nmod / xyratio)
    }
    mlon    <- seq(min(vmod.x),max(vmod.x),length.out = nx)
    mlat    <- seq(min(vmod.y),max(vmod.y),length.out = ny)
    lon.ext <- mlon
    lat.ext <- mlat
    nlon    <- length(mlon)
    nlat    <- length(mlat)
    lon <- t(t(replicate(nlat, mlon)))
    lat <- t(replicate(nlon, mlat))
    
 } else {
      if (!is.element(geo.coordinate.names[1], names(nc$var)) || !is.element(geo.coordinate.names[2], names(nc$var))) {
       if (!is.element(geo.coordinate.names[1], names(nc$dim)) || !is.element(geo.coordinate.names[2], names(nc$dim))) stop (paste('ERROR: coordinates not found: ',geo.coordinate.names,'\n  ',sep=""))
      }
       # read in the geographic coordinates of the mapped grid
      lon    <- ncvar_get(nc, geo.coordinate.names[1])
      lat    <- ncvar_get(nc, geo.coordinate.names[2])
  
    if  (is.null(ncol(lon)) || is.na(ncol(lon))) {
     #... lat/lon are 1D. Therefore assuming geographical lat/lon. No conversion necessary in this case.
      nlon <- length(lon)
      nlat <- length(lat)
      lon.ext <- lon
      lat.ext <- lat
      mlon <- lon
      mlat <- lat
      mlon.ext <- lon
      mlat.ext <- lat
      lon <- matrix(lon, nrow=nlon, ncol=nlat)
      lat <- matrix(lat, nrow=nlon, ncol=nlat, byrow=TRUE)
    } else {
    #... lon/lat are 2D. Grid mapping is assumed in this case.
      nlon    <- dim(lon)[1]
      nlat    <- dim(lon)[2]
      mlon    <- seq(1,nlon)
      mlat    <- seq(1,nlat)
      # extend the lon and lat matrices by one at each side
      mlon.ext <- seq(1,nlon+2)
      mlat.ext <- seq(1,nlat+2)
      lon.ext<-matrix(0, nrow=(nlon+2), ncol=(nlat+2))
      lat.ext<-matrix(0, nrow=(nlon+2), ncol=(nlat+2))
      lon.ext[2:(nlon+1), 2:(nlat+1)] <- lon
      lat.ext[2:(nlon+1), 2:(nlat+1)] <- lat
      lon.ext[1,2:(nlat+1)]        <- 2*lon[1,1:nlat] -lon[2,1:nlat]
      lon.ext[nlon+2,2:(nlat+1)]   <- 2*lon[nlon,1:nlat] -lon[nlon-1,1:nlat]
      lon.ext[2:(nlon+1),1]        <- 2*lon[1:nlon,1] -lon[1:nlon,2]
      lon.ext[2:(nlon+1),(nlat+2)] <- 2*lon[1:nlon,nlat] -lon[1:nlon,(nlat-1)]
      lon.ext[1,1]                 <- 2*lon.ext[1,2] -lon.ext[1,3]
      lon.ext[1,(nlat+2)]          <- 2*lon.ext[1,(nlat+1)] -lon.ext[1,nlat]
      lon.ext[(nlon+2),1]          <- 2*lon.ext[(nlon+2),2] -lon.ext[(nlon+2),3]
      lon.ext[(nlon+2),(nlat+2)]   <- 2*lon.ext[(nlon+2),(nlat+1)] -lon.ext[(nlon+2),nlat]
  
      lat.ext[1,2:(nlat+1)]        <- 2*lat[1,1:nlat] -lat[2,1:nlat]
      lat.ext[nlon+2,2:(nlat+1)]   <- 2*lat[nlon,1:nlat] -lat[nlon-1,1:nlat]
      lat.ext[2:(nlon+1),1]        <- 2*lat[1:nlon,1] -lat[1:nlon,2]
      lat.ext[2:(nlon+1),(nlat+2)] <- 2*lat[1:nlon,nlat] -lat[1:nlon,(nlat-1)]
      lat.ext[1,1]                 <- 2*lat.ext[1,2] -lat.ext[1,3]
      lat.ext[1,(nlat+2)]          <- 2*lat.ext[1,(nlat+1)] -lat.ext[1,nlat]
      lat.ext[(nlon+2),1]          <- 2*lat.ext[(nlon+2),2] -lat.ext[(nlon+2),3]
      lat.ext[(nlon+2),(nlat+2)]   <- 2*lat.ext[(nlon+2),(nlat+1)] -lat.ext[(nlon+2),nlat]
    }
  }
    #---------------------------------------------------------   
    
    # check for orography and land sea mask data
    noread  <- c("lon", "lat")
    var.n   <- setdiff(names(nc$var), noread)
    which.v<-sapply(nc$var[names(nc$var) %in% var.n], function(x) all(ncatt_get(nc,x,"standard_name")$value == "surface_altitude"))
    varnames<-names(which.v)[which(which.v)]
    if (length(varnames)>0) {
      if (length(varnames) > 1) {
        oro.name <- names(which.v)[which.max(which.v)]
      } else {
        oro.name = varnames
      }
    } else {
        oro.name = NA
    }

    which.v<-sapply(nc$var[names(nc$var) %in% var.n], function(x) all(ncatt_get(nc,x,"standard_name")$value == "land_area_fraction"))
    varnames<-names(which.v)[which(which.v)]
    if (length(varnames)>0) {
      if (length(varnames) > 1) {
        lsm.name <- names(which.v)[which.max(which.v)]
      } else {
        lsm.name = varnames
      }
    } else {
        lsm.name = NA
    }

    #---------------------------------------------------------   
    
    # apply scaling factor and offset
    data    <- ncvar_get(nc, varname) * scale.factor + add.offset
 #print(dim(data))

    #---------------------------------------------------------   

    # select timestep i.time and/or vertical level i.lev    
    if (icon) {
      if (length(dim(data)) == 2){
       if (length(nc$var[[varname]]$size) == 2){
          if (ncatt_get(nc, nc$var[[varname]]$dim[[3]]$name, "standard_name")$value == 'time' && nc$var[[varname]]$dim[[2]]$len != 1 ) {
            data    <- data[,i.time]
          } else {
            data    <- data[,i.lev]
          }
        } else if (length(nc$var[[varname]]$size) == 3){
         if (ncatt_get(nc, nc$var[[varname]]$dim[[3]]$name, "standard_name")$value == 'time'  && nc$var[[varname]]$dim[[3]]$len != 1) {
          data    <- data[,i.time]
          } else {
           data    <- data[,i.lev]
          }
        }
      } else if (length(dim(data)) == (3)){
        data    <- data[,i.lev,i.time]
      }
    } else {   
    if (length(dim(data)) == 3){
     if (length(nc$var[[varname]]$size) == 3){
        if (ncatt_get(nc, nc$var[[varname]]$dim[[3]]$name, "standard_name")$value == 'time' && nc$var[[varname]]$dim[[3]]$len != 1 ) {
          data    <- data[,,i.time]
        } else {
          data    <- data[,,i.lev]
        }
      } else if (length(nc$var[[varname]]$size) == 4){
        if (ncatt_get(nc, nc$var[[varname]]$dim[[3]]$name, "standard_name")$value == 'time'  && nc$var[[varname]]$dim[[4]]$len != 1) {
          data    <- data[,,i.time]
        } else {
          data    <- data[,,i.lev]
        }
      }
    } else if (length(dim(data)) == 4){
       data    <- data[,,i.lev,i.time]
    }
  }
    #---------------------------------------------------------   

    # store information on variables
    if (is.null(longname)) longname  <- nc$var[[varname]]$longname
    if (is.null(units))    units     <- nc$var[[varname]]$units
    flag_values <- ncatt_get(nc, varname, "flag_values")
    if (flag_values$hasatt){
        flag_values <- flag_values$value
    } else {
        flag_values <- NULL
    }
    flag_meanings <- ncatt_get(nc, varname,"flag_meanings")
    if (flag_meanings$hasatt){
        flag_meanings <- unlist(strsplit(flag_meanings$value, " "))
    } else {
        flag_meanings <- NULL
    }
    
    #---------------------------------------------------------   
    # close the netCDF file
    nc_close(nc)

    #---------------------------------------------------------   
    # mask out the sponge zone
    if (!is.na(sponge) & sponge > 0) {
        data[c(1:sponge,(nlon-sponge+1):nlon),] <- NA
        data[,c(1:sponge,(nlat-sponge+1):nlat)] <- NA
    }

    #---------------------------------------------------------   
    # if HSURF is plotted, the lsm.file is the same as file
    if (!is.na(oro.name) && varname == oro.name && !is.na(lsm.name)){
        lsm.file    <- file
    }

    #---------------------------------------------------------   
    # read in the land sea mask
       if (is.null(lsm.file) || is.na(lsm.file)){
      ##      no sea mask. Set all lsm values to TRUE, i.e. land everywhere
        if (icon) {
          lsm <- !vector(mode="logical",length=length(data))
        } else {
        lsm         <- array(TRUE, dim(data))
        }
        alt.contour <- FALSE
    } else {
        nc.lsm  <- nc_open(lsm.file)
        # check for orography and land sea mask data
        noread  <- c("lon", "lat")
        which.v<-sapply(nc$var[names(nc$var) %in% var.n], function(x) all(ncatt_get(nc,x,"standard_name")$value == "land_area_fraction"))
        varnames<-names(which.v)[which(which.v)]
        if (length(varnames)>0) {
          if (length(varnames) > 1) {
            lsm.name <- names(which.v)[which.max(which.v)]
          } else {
            lsm.name = varnames
          }
        } else {
          lsm.name = NA
        }
        if (is.na(lsm.name)) {
          if (icon) {
            lsm <- !vector(mode="logical",length=length(data))
          } else {
            lsm         <- array(TRUE, dim(data))
          } 
        } else {
          lsm     <- ncvar_get(nc.lsm, lsm.name)
          lsm     <- lsm > 0.5
          if (!icon) {
            if (any(names(nc.lsm$var) %in% c("lon", "lat"))){
                lon <- ncvar_get(nc.lsm, "lon")
                lat <- ncvar_get(nc.lsm, "lat")
            } else {
                tmp <- rot2geo(pollon, pollat, rep(rlon, nlat), rep(rlat, each=nlon), polgam)
                lon <- array(tmp$x, c(nlon,nlat))
                lat <- array(tmp$y, c(nlon,nlat))
            }
          }
        if (alt.contour & any(names(nc.lsm$var) == lsm.name)){
          if (any(names(nc.lsm$var) == lsm.name)){
              alt <- ncvar_get(nc.lsm, lsm.name)
        }
        }
        }
        nc_close(nc.lsm)
    }

      if (!icon) { 
      if (lon[1,1] > lon[nrow(lon),1] & lon[1,1] < lon[2,1]){
          lon[lon < 0] <- lon[lon < 0] + 360
      }
      }
    
    #---------------------------------------------------------   
    # set plot levels
    if (is.null(levels)){
        if (is.na(oro.name)) oro.name<-'NONE'
        if (varname == oro.name){
            levs <- c(-200,0,100,200,500,1000,1500,2000,3000,10000)
        } else {
            if (is.null(flag_values)){
              if (!is.null(col)) {
               levs    <- pretty(data, length(col)+1)
            } else {
               levs    <- pretty(data, 20)
              }
            } else {
            flags<-data.frame(flag_values=flag_values, flag_meanings=flag_meanings)
            flag_values<-as.vector(flags[order(flags$flag_values),][,1])
            flag_meanings<-as.vector(flags[order(flags$flag_values),][,2])
            levs    <- approx(seq(along=flag_values), flag_values, 
                0.5 + 0:length(flag_values), yleft=2*flag_values[1]-flag_values[2],
                yright=2*flag_values[length(flag_values)]-flag_values[length(flag_values)-1])$y
            }
        }
    } else {
        levs        <- levels
    }

    #---------------------------------------------------------   
    # set the colours and plot levels   
    ncols <- length(levs)-1
    if (is.null(col)){
        if (colourplot){
           if (is.na(oro.name)) oro.name<-'NONE'
            if (varname == oro.name){
                colours	<- .colseq(length(levs)-1, .hsurf, smooth=0)
                sea.col <- .water
            } else {
                colours     <- rbfun(ncols)
            }
        } else {
            colours <- grey((ncols+1):1/(ncols+1))[2:(ncols+1)]
            sea.col <- "white"
        }
    } else {
        colours <- rep(col, length.out=ncols)
    }

    #---------------------------------------------------------   
    # get the coast lines and political boundary data
        if (hires){
        worlddb <- "worldHires"
        if (any(lon > 180)) worlddb <- "worldHires2"
    } else {
        worlddb <- "world"
        if (any(lon > 180)) worlddb <- "world2"
    }
    if (alt.poli & interior){
      data(polibound)
      world <- polibound
      if (any(lon > 180)){
          world$x[!is.na(world$x) < 0] <- world$x[!is.na(world$x) < 0] + 360
      }
      for (add.name in c(".*Lake.*", ".*Sea.*")){
          world.add <- try(map(worlddb, region=add.name, plot=F, xlim=range(lon), ylim=range(lat)), silent=TRUE)
          if (class(world.add) != 'try-error' & length(world.add) > 0){
              world   <- list(x=c(world$x, NA, world.add$x),
                              y=c(world$y, NA, world.add$y))
        }
      }
       
    } else {
        world       <- map(worlddb,interior=interior, plot=F, 
                         xlim=range(lon), ylim=range(lat))
    }
    if (!interior){
        # remove Lesotho and add the Lakes and Seas
        for (add.name in c(".*Lake.*", ".*Sea.*", ".*Island.*")){
            world.add <- try(map(worlddb, region=add.name, plot=F, 
                xlim=range(lon), ylim=range(lat)), silent=TRUE)
            if (class(world.add) != 'try-error' & length(world.add) > 0){
                world   <- list(x=c(world$x, NA, world.add$x),
                    y=c(world$y, NA, world.add$y), 
                    names=c(world$names, world.add$names))
            }
        }
        world.remove<- map(worlddb, region="Lesotho", plot=F)
        ind.i       <- which(world$x %in% world.remove$x & world$y %in% world.remove$y)
        world$x[ind.i] <- NA
        world$y[ind.i] <- NA
    }
     world.rot   <- geo2mod(lon.ext, lat.ext, world$x, world$y ,gridmapping)


     #---------------------------------------------------------   
     # get give river and lake data
     if (rivers){
        riv.dat     <- map('rivers', plot=F)
        rivers.rot  <- geo2mod(lon.ext, lat.ext, riv.dat$x, riv.dat$y ,gridmapping)

    } 

    if (lakes){
        lak.dat     <- map('lakes', plot=F)
        lakes.rot  <- geo2mod(lon.ext, lat.ext, lak.dat$x, lak.dat$y ,gridmapping)

    } 

     #---------------------------------------------------------   
     # plot the data
    data.tmp	    <- data
    if (icon) {
      plot(vmod.xrange,vmod.yrange,type='n',xlim=vmod.xrange, ylim=vmod.yrange, xaxs = "i", yaxs = "i",xaxt='n', yaxt='n',main=main, xlab=xlab, ylab=ylab, cex.main=cex.main)
      if (is.null(col)){
        polygon.colours<-colours[findInterval(data.tmp,levs)]
      } else {
        polygon.colours<-col[findInterval(data.tmp,levs)]
      }
      vlen<-dim(vlon)[2]
     for (i in 1:vlen) {
#        polygon(vmod.x[,i],vmod.y[,i],col=colors[i],border=NA)
        polygon(vmod.x[,i],vmod.y[,i],col=polygon.colours[i],border=polygon.colours[i])
      }

    } else {

    data.tmp[!lsm]  <- NA
    if (smoothed) { 
      image(mlon, mlat, matrix(rep(NA,length(mlon)*length(mlat)), nrow=length(mlon), ncol=length(mlat)), breaks=levs, axes=F, add=add,
        col=colours, main=main, xlab=xlab, ylab=ylab,
        cex.axis=cex.axis, cex.lab=cex.lab, cex.main=cex.main)
      .filled.contour(mlon, mlat, data.tmp, levs, col = colours)
     } else {
      image(mlon, mlat, data.tmp, breaks=levs, axes=F, add=add,
        col=colours, main=main, xlab=xlab, ylab=ylab,
        cex.axis=cex.axis, cex.lab=cex.lab, cex.main=cex.main)
     }
    }


    if (!is.na(any(!lsm)) & !is.null(sea.col)){
        ## sea points with sea.col
        data.tmp	<- data
        data.tmp[lsm]   <- NA
        if (icon) {
          vrotx.lsm<-vrot$x[,-which(apply(vrot$x,2,function(x)all(is.na(x))))]
          vroty.lsm<-vrot$y[,-which(apply(vrot$y,2,function(x)all(is.na(x))))]
          lsmlen<-dim(vrotx.lsm)[2]
          for (i in 1:lsmlen) {
            polygon(vrotx.lsm[,i],vroty.lsm[,i],col=sea.col,border=NA)
          }
       
        } else {

        image(mlon, mlat, data.tmp, breaks=c(-1e10,1e10), 
            col=sea.col, add=T, axes=F, xlab="", ylab="")
    }    
    }    

    if (!icon && !is.null(na.col)){
        ## NA points with na.col
        data.tmp	<- data
        data.tmp[is.na(data)]   <- 0
        data.tmp[!is.na(data)]   <- NA
        image(mlon, mlat, data.tmp, breaks=c(-1e10,1e10), 
            col=na.col, add=T, axes=F, xlab="", ylab="")
    }    

#    if (!icon && exists("alt")){
#        if (is.null(alt.lev)) alt.lev <- pretty(alt, 10)
#        contour(mlon, mlat, alt, lev=alt.lev, drawlabels=F, add=T)
#    }
    if (exists("alt")){
        if (icon) {
          vlen=dim(vlon)[2] 
          for (i in 1:vlen) {  
            if (alt[i] >= 0.5) {polygon(vlon[,i],vlat[,i],col=NA,border='black',lwd=0.2)} 
          } 
        } else {
          if (is.null(alt.lev)) alt.lev <- pretty(alt, 10)
          contour(mlon, mlat, alt, lev=alt.lev, drawlabels=F, add=T)
        }
    }

     #---------------------------------------------------------  
     #plot rivers and lakes
     
     if (rivers){
        riv.col <- sea.col
        if (is.null(riv.col)) riv.col <- "white"
        lines(rivers.rot$x, rivers.rot$y, col=riv.col)
    }
    if (lakes){
        lak.col <- "black"
#        if (is.null(lak.col)) lak.col <- "white"
        lines(lakes.rot$x, lakes.rot$y, col=lak.col)
    }

    if (!exists("alt")){
        ##lon-lat-lines
  
        #... prevent straight lines across the plot
        for (i in 2:length(world.rot$x)) {
          if (!is.na(world.rot$x[i]) &  !is.na(world.rot$x[i-1])) {
            if (abs(world.rot$x[i]-world.rot$x[i-1]) > abs(mlon[nlon]-mlon[1])) world.rot$x[i]=NA
          }
        }
        lines(world.rot$x, world.rot$y, lwd=map.lwd, lty=map.lty)
    }

     #---------------------------------------------------------   
     # plot cities
     
     if (cities){
        # some cities
        data(world.cities)
        # compute rotated coordinates
        coords <- geo2mod(lon.ext, lat.ext,world.cities$long,world.cities$lat,gridmapping)
        world.cities$mlon <- coords$x
        world.cities$mlat <- coords$y
        
        # select the cities within the plot area
        region.cities   <- world.cities[world.cities$mlon > min(mlon) &
            world.cities$mlat > min(mlat) &
            world.cities$mlon < max(mlon) &
            world.cities$mlat < max(mlat),]
        
        # further select the cities according to minpop
        # or ncities
        if (is.null(minpop)){
            region.cities   <- region.cities[order(region.cities$pop, 
                decreasing=TRUE),]
            region.cities   <- region.cities[1:ncities,]
        } else {
            region.cities   <- region.cities[region.cities$pop > minpop,]            
        }
        # their point 
        points(region.cities$mlon,region.cities$mlat, pch=city.pch)
        # their label
        if (label){
            text(region.cities$mlon, region.cities$mlat,
                labels=region.cities$name, offset=0.5, pos=3, cex=cex.txt)
        }
    }

    
    box(lwd=1)
    
    #---------------------------------------------------------   
    # plot grid and grid labels
    
    # check whether lon and lat are present, else set grid to FALSE
    if (!(exists("lon") & exists("lat"))){
        grid    <- FALSE
    }
    
    if (grid){

        if (missing(lon.ind)){
            lon.ind	<- pretty(lon,nlongrid)
        }
        if (missing(lat.ind)){
            lat.ind <- pretty(lat,nlatgrid)
        }

        lat.seq<-seq(max(-89.5,min(lat)),min(89.5,max(lat)), by=.2)
        for (i in 1:length(lon.ind)) {
          lon.seq<-rep(lon.ind[i],times=length(lat.seq))
          rotated.poly<-geo2mod(lon.ext, lat.ext, lon.seq, lat.seq,gridmapping)
          lines(rotated.poly,lty=grid.lty, lwd=grid.lwd)
        }
        contour(mlon, mlat, lat, levels=lat.ind, 
            lty=grid.lty, lwd=grid.lwd, drawlabels=F, axes=F, add=T)

        if (any(grid.txt)){
            if (is.na(grid.txt[2])) grid.txt[2:4] <- grid.txt[1]
            lon.ind2 <- lon.ind
            if (any(lon > 180)) lon.ind2[lon.ind > 180] <- lon.ind[lon.ind > 180] - 360
            lon.txt <- paste(abs(lon.ind2), '*degree', c('~ W', '', '~ E')[sign(lon.ind2) + 2])
            lab.w <- strwidth(parse(text=lon.txt), cex=cex.axis)
            lon.i <- apply(as.matrix(lon.ind), 1, function(x) if (x > min(lon[,1]) & x < max(lon[,1])) which.min((lon[,1]-x)**2) else NA)
            lon.at <- geo2mod(lon.ext, lat.ext,lon[lon.i,1], lat[lon.i,1], gridmapping)$x 
            for (i in (min(which(!is.na(lon.at)))+1):max(which(!is.na(lon.at)))){
                if (all(is.na(lon.at[1:(i-1)]))) {
                  lo.i <- NA
                } else {
                  lo.i <- max(which(!is.na(lon.at[1:(i-1)])))
                }
                if (!is.na(lon.at[i]) && !is.na(lo.i)) {
                  dist <- lon.at[i] - lon.at[lo.i]
                  if (dist < 0.6*(lab.w[i] + lab.w[lo.i])) lon.at[i] <- NA
                } else {
                  lon.at[i] <- NA
                }
                if (all(is.na(lon.at))) {
                  grid.txt[1] = F
                }
            }
            if (grid.txt[1]) axis(1, at=lon.at, labels=parse(text=lon.txt), tick=F, line=-0.5, cex.axis=cex.axis)
            lon.i <- apply(as.matrix(lon.ind), 1, function(x) if (x > min(lon[,ncol(lon)]) & x < max(lon[,ncol(lon)])) which.min((lon[,ncol(lon)]-x)**2) else NA)
            lon.at <- geo2mod(lon.ext, lat.ext, lon[lon.i,ncol(lon)], lat[lon.i,ncol(lon)], gridmapping)$x 
            for (i in (min(which(!is.na(lon.at)))+1):max(which(!is.na(lon.at)))){
                lo.i <- max(which(!is.na(lon.at[1:(i-1)])))
                dist <- lon.at[i] - lon.at[lo.i]
                if (!is.na(lon.at[i])) {
                  if (dist < 0.6*(lab.w[i] + lab.w[lo.i])) lon.at[i] <- NA
                }
            }
            if (grid.txt[3]) axis(3, at=lon.at, labels=parse(text=lon.txt), tick=F, line=-0.5, cex.axis=cex.axis)
            
            lat.txt <- paste(abs(lat.ind), '*degree', c('~ S', '', '~ N')[sign(lat.ind) + 2])
            lab.w <- strheight(parse(text=lat.txt), cex=cex.axis)
            lat.i <- apply(as.matrix(lat.ind), 1, function(x) if (x > min(lat[,1]) & x < max(lat[1,])) which.min((lat[1,]-x)**2) else NA)
            lat.at <- geo2mod(lon.ext, lat.ext, lon[1,lat.i], lat[1,lat.i],gridmapping)$y
            for (i in (min(which(!is.na(lat.at)))+1):max(which(!is.na(lat.at)))){
                lo.i <- max(which(!is.na(lat.at[1:(i-1)])))
                dist <- lat.at[i] - lat.at[lo.i]
                if (dist < (lab.w[i] + lab.w[lo.i])) lat.at[i] <- NA
            }
            if (grid.txt[2]) axis(2, at=lat.at, labels=parse(text=lat.txt), tick=F, line=-0.5, cex.axis=cex.axis, las=1)
            lat.i <- apply(as.matrix(lat.ind), 1, function(x) if (x > min(lat[nrow(lat),]) & x < max(lat[nrow(lat),])) which.min((lat[nrow(lat),]-x)**2) else NA)
            lat.at <- geo2mod(lon.ext, lat.ext, lon[nrow(lat),lat.i], lat[nrow(lat),lat.i], gridmapping)$y
            for (i in (min(which(!is.na(lat.at)))+1):max(which(!is.na(lat.at)))){
#                lo.i <- max(which(!is.na(lat.at[1:(i-1)])))
#                dist <- lat.at[i] - lat.at[lo.i]
#               if (dist < (lab.w[i] + lab.w[lo.i])) lat.at[i] <- NA
                if (all(is.na(lat.at[1:(i-1)]))) {
                  lo.i <- NA
                } else {
                  lo.i <- max(which(!is.na(lat.at[1:(i-1)])))
                }
                if (!is.na(lat.at[i]) && !is.na(lo.i)) {
                  dist <- lat.at[i] - lat.at[lo.i]
                  if (dist < 0.6*(lab.w[i] + lab.w[lo.i])) lat.at[i] <- NA
                } else {
                  lat.at[i] <- NA
                }
                if (all(is.na(lat.at))) {
                  grid.txt[4] = F
                }
            }
            if (grid.txt[4]) axis(4, at=lat.at, labels=parse(text=lat.txt), tick=F, line=-0.5, cex.axis=cex.axis, las=1)
        }
    }

    #---------------------------------------------------------   
    # create the return list
    
    out         <- list(col=colours, lev=levs, sea.col=sea.col, sea.lab=sea.lab, flag_values=flag_values,
        flag_meanings=flag_meanings, longname=longname, units=units, gridmapping=gridmapping)
    class(out)  <- "plotmap"
    invisible(out)

}

