#' Create a mask from a given closed polygon
#' 
#'   This function reads a closed polygon given as geographical longitude and latitude pairs.
#'   The inner part of the polygon is given the value 1. The value in the outer part can be chosen by the user. This value will be interpreted as undefined in netCDF.
#'   Optionally the created mask can be written into a file
#'   
#' @param Cfile NetCDF input file containing the information of the domain
#' @param Pfile ASCII input file containing the polygon points in geographical coordinates. One longitude latitude pair per line.
#' @param Mfile Output file receiving the mask
#' @param multiply allows for fractional grid box values in the vicinity of the polygon line. \code{multiply} is the factor by which the grid is inflated temporarily during the computation. 1 = no fraction, 2 = halfs, 3 = thirds, 4 = quarters, etc.
#' @param skiplines lines to be skipped when reading from Pfile
#' @param undef value in the outer part of the polygon
#' 
#' @return Returns a matrix of dimension nlon x nlat of fractional gridbox area within the polygon.
#' 
#' @note Multiple polygons in a file must be divided by lines holding "-999 -999"
#' 
#' @author Burkhardt Rockel (HZG)
#' 
#' @examples   
#' # create a mask with fractional grid box coverage and skip the first line when reading the polygon file
#' polymask (Cfile="/path/lffd2001010100.nc", Pfile="/path/polygon.txt", Mfile="/path/mask.nc", multiply=4, skiplines=1)
#' 
#' @export


polymask<-function(Cfile, Pfile, Mfile=NULL, multiply=1, skiplines=0, undef=-1.E20, ...) {

#
#  return value of the funcion is the 2D-mask
#

# read longitude and latitude data 
nc <- nc_open(Cfile)
if ( is.null(nc$var$rotated_pole$id) ) {
  # normal geographical grid
  mlon <- nc$dim$lon$vals
  mlat <- nc$dim$lat$vals
} else {
  # rotated grid
    # read in the coordinates of the rotated pole
    pollon  <- as.numeric(ncatt_get(nc, "rotated_pole", "grid_north_pole_longitude")$value)
    pollat  <- as.numeric(ncatt_get(nc, "rotated_pole", "grid_north_pole_latitude")$value)
    polgam  <- ncatt_get(nc, "rotated_pole", "north_pole_grid_longitude")
    if (polgam$hasatt){
        polgam <- polgam$value
    } else {
        polgam <- 0
    }
    mlon    <- nc$dim$rlon$vals
    mlat    <- nc$dim$rlat$vals

    lon <- ncvar_get(nc, "lon")
    lat <- ncvar_get(nc, "lat")

}
nc_close(nc)

# read polygon data
tmp <- read.table(Pfile, skip=skiplines)

# find end points of closed polygons
fnd999 <- which(tmp[,1] == -999)
if (length(fnd999)==0) { 
   print("Single Polygon file") 
   poly.ends<-which(duplicated(tmp))
   poly.starts<-poly.ends - poly.ends[1] + 1
   poly.number <- 1
} else {
   poly.ends <- c(fnd999 - 1,nrow(tmp))
   poly.starts <- c(1,fnd999 + 1)
   poly.number <- length(poly.ends)
}

# loop over number of closed polygons
for (npoly in 1:poly.number) {

poly.start<-poly.starts[npoly]
poly.end<-poly.ends[npoly]

plon<-tmp[,1][poly.start:poly.end]
plat<-tmp[,2][poly.start:poly.end]

plon[length(plon)+1] <- plon[1]
plat[length(plat)+1] <- plat[1]

# transform polygon data onto rotated grid, if necessary
if ( ! is.null(nc$var$rotated_pole$id) ) {
  poly <- geo2rot(pollon, pollat, plon, plat, polgam)
  plon <- poly$x
  plat <- poly$y
}

# determine the polygon area
  outgrid.tmp <- frac_in_polygon(mlon, mlat, plon, plat, multiply = multiply)
  if (npoly == 1 ) {
    outgrid <- outgrid.tmp
  }
  else
  {
    outgrid <- outgrid + outgrid.tmp
  }
  
}

outgrid[outgrid > 1] <-1
outgrid[outgrid==0] <- undef

if ( ! is.null(Mfile) ) {

if ( is.null(nc$var$rotated_pole$id) ) {
    mlondim <- ncdim_def("lon", "degrees_east", mlon, longname="longitude")
    mlatdim <- ncdim_def("lat", "degrees_north", mlat, longname="latitude")
    var <- ncvar_def("MASK", "0-1", list(mlondim, mlatdim), undef,longname="mask",prec="float")
    ncid <- nc_create(Mfile, var)
    ncatt_put(ncid,"lon","standard_name","longitude",prec="text")
    ncatt_put(ncid,"lon","standard_name","latitude",prec="text")
    ncatt_put(ncid,var,"_FillValue",undef,prec="float")
    ncatt_put(ncid,0,"Conventions","CF-1.0",prec="text")
    ncvar_put(ncid, var, outgrid)
    nc_close(ncid)
#    system(paste('ncwa -h -O -a rotated_pole ',Mfile,' ',Mfile,sep="")) # to make rotated_pole a scalar

} else {

    mlondim <- ncdim_def("rlon", "degrees", mlon,longname="longitude in rotated pole grid")
    mlatdim <- ncdim_def("rlat", "degrees", mlat,longname="latitude in rotated pole grid")
    poledim<- ncdim_def("rotated_pole","", 1:1, create_dimvar=FALSE )
    londef <- ncvar_def("lon", "degrees_east",list(mlondim,mlatdim),longname="longitude",prec="float")
    latdef <- ncvar_def("lat", "degrees_north",list(mlondim,mlatdim),longname="latitude",prec="float")
    gmap <- ncvar_def("rotated_pole", "", poledim, prec="char")
    var <- ncvar_def("MASK", "0-1", list(mlondim, mlatdim), undef,longname="mask",prec="float")
    ncid <- nc_create(Mfile, list(londef,latdef,gmap,var))
    ncatt_put(ncid,"rlon","standard_name","grid_longitude",prec="text")
    ncatt_put(ncid,"rlat","standard_name","grid_latitude",prec="text")
    ncatt_put(ncid,londef,"standard_name","longitude",prec="text")
    ncatt_put(ncid,latdef,"standard_name","latitude",prec="text")
    ncatt_put(ncid,gmap,"grid_mapping_name","rotated_latitude_longitude",prec="text")
    ncatt_put(ncid,gmap,"grid_north_pole_longitude",pollon,prec="float")
    ncatt_put(ncid,gmap,"grid_north_pole_latitude",pollat,prec="float")
    if (polgam != 0.) {
      ncatt_put(ncid,gmap,"north_pole_grid_longitude",polgam,prec="float")
    }
    ncatt_put(ncid,var,"coordinates","lon lat",prec="text")
    ncatt_put(ncid,var,"grid_mapping","rotated_pole",prec="text")
    ncatt_put(ncid,var,"_FillValue",undef,prec="float")
    ncatt_put(ncid,0,"Conventions","CF-1.0",prec="text")
    ncvar_put(ncid, londef, lon)
    ncvar_put(ncid, latdef, lat)
    ncvar_put(ncid, var, outgrid)
    nc_close(ncid)
#    system(paste('ncwa -h -O --no_cll_mth -a rotated_pole ',Mfile,' ',Mfile,sep="")) # to make rotated_pole a scalar
}

}

  return("Everything ok")

}  # end of function polymask

