#' Create a vector plot
#' 
#' This function plots wind vectors over a given plot.
#' 
#' @param u,v 2D arrays holding the values for the velocity components in x and y direction
#' @param x,y 1D arrays holding the coordinate values in x and y direction
#' @param scale scaling of the vector length
#' @param length length of the edges of the arrow head (in inches)
#' @param units units of u and v
#' @param lwd line width
#' @param colvar 2D array of a quantity to be plotted as colors of the vector
#' @param col colors of the vectors
#' @param levels levels for the colors
#' @param n plot every n's vector
#' @param legend.scale sets the scale of the vector that is plotted over the top right of the image
#' 
#' @return  There is no return value
#' 
#' @author Burkhardt Rockel (HZG)
#' 
#' @examples
#'   # plot vectors of 10m wind over an image created with plotmap
#'   col<-c("#FFFF82","#F0F000","#82FF82","#00FF00","#00C800","#009600","#00A0FF",
#'          "#0064FF","#0000FF","#8C1EDC","#A000A0","#BE0082","#BE005A","#BE0000")
#'   levs<-seq(-23.,23.,2.)
#'   nc<-nc_open(ncfile)
#'   rlon <- ncvar_get(nc,"rlon")
#'   rlat <- ncvar_get(nc,"rlat")
#'   u <- ncvar_get(nc,"U_10M")
#'   v <- ncvar_get(nc,"V_10M")
#'   t <- ncvar_get(nc,"T_2M")
#'   units <- paste("10 ",ncatt_get(ncu, "U_10M", "units")$value,sep="")
#'   nc_close(nc)
#'   tmp <- plotmap(.....)
#'   # with colors
#'   windvec(u, v, rlon, rlat,units=units,colvar=t,col=col,levels=levs,legend.scale=10)
#'   # without colors
#'   windvec(u, v, rlon, rlat,units=units,legend.scale=10)
#'   
#' @export

windvec <- function(u, v, x, y, scale=2., length=0.05, units=NA, lwd=1, colvar=NULL, col=par("fg"), levels=1, n=5, legend.scale=1){
   x1 <- x[c(n*1:(length(x)/n))]
   y1 <- y[c(n*1:(length(y)/n))]
   u1 <- u[c(n*1:(length(x)/n)),c(n*1:(length(y)/n))]
   v1 <- v[c(n*1:(length(x)/n)),c(n*1:(length(y)/n))]
   lo <- rep(x1, length(u1))
   la <- rep(y1, each=length(u1)/length(y1))
   w <- sqrt(u1**2 + v1**2)
   wmax <- max(w, na.rm=TRUE)
   if (missing(scale)) scale <- 3*median(abs(c(diff(x1), diff(y1))))
   uu <- as.vector(u1)/wmax*scale
   vv <- as.vector(v1)/wmax*scale
   if ( is.null(colvar) ) {
     arrows(lo, la, lo+uu, la+vv, length=length,lwd=lwd,col=col)
   } else {
     arrows(lo, la, lo+uu, la+vv, length=length,lwd=lwd,col=col[findInterval(colvar,levels)])
   }
   # legend
    if (!is.na(units)) {
      usr <- par( "usr" )
      par(xpd=NA)
      arrows(usr[ 2 ]-(legend.scale/wmax*scale),usr[ 4 ]+xinch(0.1),usr[ 2 ],usr[ 4 ]+xinch(0.1), length=length,lwd=lwd)
      text(usr[ 2 ], usr[ 4 ]+xinch(0.3),units, adj = c( 1, 1 ))
    par(xpd=FALSE)
   }
}
